INCOME_HIERARCHY = {
    "earned": ["salary", "freelance", "rideshare & delivery", "donations"],
    "passive": [
        "child support",
        "social security",
        "unemployment insurance",
        "government benefits",
        "long term rent",
        "short term rent",
        "ecommerce",
        "donations",
        "interest and dividends",
        "investment",
        "retirement funds",
    ],
}
INCOME_CLASSES = list(INCOME_HIERARCHY.keys())
INCOME_LABELS = [
    label for class_labels in list(INCOME_HIERARCHY.values()) for label in class_labels
]

UNDETERMINED_LABEL = "possible income - please verify"


class IncomeReport:
    def __init__(self, income_report_dict):
        self.data = income_report_dict
        self.income_groups = sorted(
            [IncomeGroup(d) for d in self.data], key=lambda x: x.amount, reverse=True
        )

    def get_income_groups(self):
        return self.igs

    def get_main_income_type(self):
        sources = {k: 0 for k in INCOME_LABELS}

        for ig in self.income_groups:
            sources[ig.income_type] += 1

        return max(sources)

    def get_income_types(self):
        sources = []
        for ig in self.income_groups:
            sources.append(ig.income_type)

        return list(set(sources))

    def get_class_summary(self, income_class):
        if income_class not in INCOME_CLASSES:
            raise RuntimeError("Unsupported income class")

        igs = []
        for ig in self.income_groups:
            if ig.income_type in INCOME_HIERARCHY[income_class]:
                igs.append(ig)

        return IncomeClassSummary(igs)

    def get_undetermined_income(self):
        igs = []
        for ig in self.income_groups:
            if ig.income_type == UNDETERMINED_LABEL:
                igs.append(ig)
        igs = sorted(igs, key=lambda x: x.amount, reverse=True)
        return igs


class IncomeGroup:
    def __init__(self, income_group_dict):
        self.data = income_group_dict

        self.amount = self.data["amount"] if "amount" in self.data else 0
        self.date_of_first_payment = (
            self.data["start_date"] if "start_date" in self.data else None
        )
        self.date_of_last_payment = (
            self.data["end_date"] if "end_date" in self.data else None
        )
        self.income_type = (
            self.data["income_type"] if "income_type" in self.data else "unknown"
        )
        self.source = self.data["source"] if "source" in self.data else "unknown"
        self.transaction_ids = (
            self.data["transaction_ids"] if "transaction_ids" in self.data else []
        )
        self.pay_frequency = (
            self.data["periodicity"] if "periodicity" in self.data else "unknown"
        )


class IncomeClassSummary:
    def __init__(self, income_groups):
        self.income_groups = income_groups

        self.amount = sum([ig.amount for ig in self.income_groups])
        self.sources = list(set([ig.source for ig in self.income_groups]))

    def __repr__(self) -> str:
        return f"{self.__class__}(amount={self.amount},sources={self.sources})"

    def __str__(self) -> str:
        return self.__repr__()



class SubscriptionReport:
    def __init__(self, subscription_management_dict):
        self.data = subscription_management_dict
        self.subscription_groups = sorted(
            [SubscriptionGroup(d) for d in self.data],
            key=lambda x: x.first_payment_date,
            reverse=True,
        )

    def get_subscription_groups(self):
        return self.subscription_groups


class SubscriptionGroup:
    def __init__(self, subscription_group_dict):
        self.data = subscription_group_dict

        self.periodicity = (
            self.data["periodicity"] if "periodicity" in self.data else "unknown"
        )
        self.price = self.data["price"] if "price" in self.data else 0
        self.merchant = self.data["merchant"] if "merchant" in self.data else "unknown"
        self.website = self.data["website"] if "website" in self.data else "unknown"
        self.logo = self.data["logo"] if "logo" in self.data else "unknown"
        self.first_payment_date = (
            self.data["first_payment_date"]
            if "first_payment_date" in self.data
            else None
        )
        self.latest_payment_date = (
            self.data["latest_payment_date"]
            if "latest_payment_date" in self.data
            else None
        )
        self.transaction_ids = (
            self.data["transaction_ids"] if "transaction_ids" in self.data else []
        )
        self.is_active = self.data["is_active"] if "is_active" in self.data else False
        self.next_expected_payment_date = (
            self.data["next_expected_payment_date"]
            if "next_expected_payment_date" in self.data
            else None
        )
        self.next_expected_payment_amount = (
            self.data["next_expected_payment_amount"]
            if "next_expected_payment_amount" in self.data
            else 0
        )

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(periodicity={self.periodicity},merchant={self.merchant}, price={self.price}, is_active={self.is_active})"



from ntropy_sdk import SDK, Transaction
from ntropy_sdk.models import CustomTransactionClassifier
from tests import API_KEY
import os
import uuid
import pytest
import sklearn
import time
import pandas as pd


tx_supermarket = Transaction(
    amount=102.04,
    description="TARGET T- 5800 20th St 11/30/19 17:32",
    entry_type="debit",
    date="2012-12-10",
    iso_currency_code="USD",
    account_holder_type="business",
    mcc=5432,
)
tx_cloud = Transaction(
    amount=24.56,
    description="AMAZON WEB SERVICES AWS.AMAZON.CO WA Ref5543286P25S Crd15",
    entry_type="debit",
    date="2012-12-10",
    account_holder_type="business",
    iso_currency_code="USD",
    mcc=1234,
)
tx_supermarket2 = Transaction(
    amount=101.04,
    description="TARGET T- 5800 20th St 11/30/19 17:32",
    entry_type="debit",
    date="2012-12-10",
    iso_currency_code="USD",
    account_holder_type="business",
    mcc=5432,
    label="supermarket",
)


@pytest.fixture
def sdk():
    sdk = SDK(API_KEY)

    url = os.environ.get("NTROPY_API_URL")
    if url is not None:
        sdk.base_url = url

    return sdk


def test_model_status(sdk):
    model_name = f"test_{str(uuid.uuid4())[:20]}"
    model = CustomTransactionClassifier(model_name=model_name, sdk=sdk)

    model.fit([tx_supermarket, tx_cloud] * 5, ["supermarket", "cloud"] * 5)
    status = model.get_status()

    assert (
        status["name"] == model_name
        and status["progress"] == 100
        and status["status"] == "ready"
    )


def test_model_fitting_sync(sdk):
    model_name = f"test_{str(uuid.uuid4())[:20]}"
    model = CustomTransactionClassifier(model_name=model_name, sdk=sdk)

    model.fit([tx_supermarket, tx_cloud] * 5, ["supermarket", "cloud"] * 5)
    assert model.predict([tx_supermarket2])[0][0] == "supermarket"
    # 1 right and 1 wrong; f1 score = 0.5
    assert (
        abs(
            model.score(
                [tx_supermarket2, tx_supermarket2], ["supermarket", "incorrect label"]
            )
            - 0.5
        )
        < 1e-6
    )


def test_model_fitting_async(sdk):
    model_name = f"test_{str(uuid.uuid4())[:20]}"
    model = CustomTransactionClassifier(model_name=model_name, sdk=sdk)

    assert model.get_params()["sync"]
    model.set_params(sync=False)
    assert not model.get_params()["sync"]

    model.fit([tx_supermarket, tx_cloud] * 5, ["supermarket", "cloud"] * 5)

    while model.get_status()["status"] == "enriching":
        time.sleep(1)

    assert model.predict([tx_supermarket2])[0][0] == "supermarket"


def test_model_fitting_df(sdk):
    txs = [tx_supermarket.to_dict(), tx_cloud.to_dict()]

    df = pd.DataFrame(txs)
    model_name = f"test_{str(uuid.uuid4())[:20]}"
    model = CustomTransactionClassifier(model_name=model_name, sdk=sdk)
    model.fit(df, ["supermarket", "cloud"])

    assert model.predict([tx_supermarket])[0][0] == "supermarket"
    assert model.predict([tx_cloud])[0][0] == "cloud"
